unit dmRPC;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, HTTPDefs, websession, fpHTTP, fpWeb, fpjsonrpc, webjsonrpc,
  SQLDB, IBConnection, fpjson;

type

  { TUserModule }

  TUserModule = class(TJSONRPCModule)
    DBConn: TIBConnection;
    Login: TJSONRPCHandler;
    CreateUser: TJSONRPCHandler;
    Logout: TJSONRPCHandler;
    QGetUser: TSQLQuery;
    DBTrans: TSQLTransaction;
    QGetNewUserID: TSQLQuery;
    QInsertUser: TSQLQuery;
    procedure CreateUserExecute(Sender: TObject; const Params: TJSONData; out
      Res: TJSONData);
    procedure DataModuleCreate(Sender: TObject);
    procedure LoginExecute(Sender: TObject; const Params: TJSONData; out
      Res: TJSONData);
    procedure LogoutExecute(Sender: TObject; const Params: TJSONData; out
      Res: TJSONData);
  private
    function Check2FA(aSeed: String; ACode: Integer): Boolean;
    Procedure ConnectDB;
    Function CheckUser(Const aName,aPassword : String; ACode : Integer) : Boolean;
    function DoCreateUser(const aName, aPassword: String): Int64;
    function GetNewUserID: Int64;
  public

  end;

var
  UserModule: TUserModule;

implementation

uses onetimepass;

{$R *.lfm}

{ TUserModule }

procedure TUserModule.LoginExecute(Sender: TObject; const Params: TJSONData;
  out Res: TJSONData);

Var
  A : TJSONArray absolute Params;
  aUserName,aPassword : String;
  aTwofactorCode : Integer;

begin
  Res:=TJSONBoolean.Create(True);
  aUserName:=A.Strings[0];
  aPassword:=A.Strings[1];
  aTwoFactorCode:=A.Integers[2];
  Res:=TJSONBoolean.Create(CheckUser(aUserName,aPassword,aTwoFactorCode));
  if Res.AsBoolean then
    Session.Variables['User']:=aUserName;
end;

procedure TUserModule.LogoutExecute(Sender: TObject; const Params: TJSONData;
  out Res: TJSONData);
begin
  Session.Variables['User']:='';
  Res:=TJSONNull.Create;
end;

procedure TUserModule.CreateUserExecute(Sender: TObject;
  const Params: TJSONData; out Res: TJSONData);

Var
  A : TJSONArray absolute Params;
  aUserName,aPassword : String;

begin
  if Session.Variables['User']<>'Admin' then
    Raise Exception.Create('Only administrators can create users');
  aUserName:=A.Strings[0];
  aPassword:=A.Strings[1];
  Res:=TJSONInt64Number.Create(DoCreateUser(aUserName,aPassword));
end;

procedure TUserModule.DataModuleCreate(Sender: TObject);
begin
  CreateSession:=True;
end;

procedure TUserModule.ConnectDB;

begin
  DBConn.UserName:='FPC';
  DBConn.Password:='Secret'; // Scramble
  DBConn.HostName:='localhost';
  DBConn.DatabaseName:='/home/firebird/pas2jsdemo.fdb';
  DBConn.Open;
  DBConn.Password:='';
end;

function TUserModule.GetNewUserID : Int64;
begin
  With QGetNewUserID do
    begin
    Open;
    try
      if Not IsEmpty then
        Result:=Fields[0].AsLargeInt
      else
        Raise Exception.Create('Failed to create an ID');
    finally
      Close;
    end;
    end;
end;

function TUserModule.DoCreateUser(const aName, aPassword: String): Int64;

Var
  aSeed : String;

begin
  aSeed:=TOTPSharedSecret();
  Result:=GetNewUserID;
  ConnectDB;
  With QInsertUser do
    begin
    ParamByName('ID').AsLargeInt:=Result;
    ParamByName('NAME').AsString:=aName;
    ParamByName('PASSWORD').AsString:=aPassword;
    ParamByName('SEED').AsString:=aSeed;
    ExecSQL;
    end;
end;

function TUserModule.CheckUser(const aName, aPassword: String; ACode: Integer): Boolean;
begin
  ConnectDB;
  With QGetUser do
    begin
    ParamByName('NAME').AsString:=aName;
    Open;
    try
      Result:=Not IsEmpty and (aPassWord=FieldByName('U_PASSWORD').AsString);
      If Result Then
        Result:=Check2FA(FieldByName('U_2FASEED').AsString,aCode);
    finally
      Close;
    end;
    end;
end;

function TUserModule.Check2FA(aSeed : String; ACode : Integer) : Boolean;

Var
  O : Integer;

begin
  Result:=TOTPValidate(aSeed,aCode,1,O);
end;


initialization
  RegisterHTTPModule('RPC', TUserModule);
end.

